class Note:
    
    def __init__(self, id = None, title = None, description = None, userId = None):
        self.Id = id
        self.Title = title
        self.Description = description
        self.UserId = userId


from dataAcces.noteDA import NoteDA
from entites.note import Note


class NoteManager:
    
    
    def Create(self, note):
        da = NoteDA()
        
        return da.Insert(note)
    
    def Update(self, note):
        da = NoteDA()
        
        return da.Update(note)
    
    def Delete(self, note):
        da = NoteDA()
        
        return da.Delete(note)
    
    def GetById(self, id):
        da = NoteDA()
        record = da.GetById(id)

        if(record != None):
            nota = Note(record[0],record[1],record[2],record[3])

        return nota

    def GetByUser(self, userId):
        da = NoteDA()
        resultDb = da.GetByUser(userId)

        notas = []
        if(resultDb != None):
            for record in resultDb:
                nota = Note(record[0],record[1],record[2],record[3])
                notas.append(nota)

        return notas




from typing import Optional


class ValidationException(Exception):
    
    def __init__(self, *args: object, field: Optional[str] == None, ) -> None:
        super().__init__(*args)
        self.Field = field



import datetime
from dataAcces.context import Context
from entites.note import Note

class NoteDA:

    def Insert(self, note):
        #MYSQL
        # sql = "INSERT INTO notas (id, titulo, descripcion, usuarioId, fecha) VALUES(NULL, %s, %s, %s, %s);"

        #SQLITE
        sql = "INSERT INTO notas (id, titulo, descripcion, usuarioId, fecha) VALUES(NULL, ?, ?, ?, ?);"

        try:
            fecha = datetime.date.today().strftime("%d/%m/%y")
            params = (note.Title, note.Description, note.UserId, fecha)
        
            context = Context()

            context.Cursor.execute(sql, params)
            context.db.commit()

            result = context.Cursor.rowcount
        except Exception as ex:
            result = 0
        return result
    
    def Update(self, note):
        #MYSQL
        # sql = "UPDATE notas SET titulo = %s, descripcion = %s, fecha = %s WHERE id = %s;"

        #SQLITE
        sql = "UPDATE notas SET titulo = ?, descripcion = ?, fecha = ? WHERE id = ?;"

        try:
            fecha = datetime.date.today().strftime("%d/%m/%y")
            params = (note.Title, note.Description, fecha, note.Id)
        
            context = Context()

            context.Cursor.execute(sql, params)
            context.db.commit()

            result = context.Cursor.rowcount
        except Exception as ex:
            result = 0
        return result
    
    def Delete(self, note):
        #MYSQL
        # sql = "DELETE FROM notas WHERE id = %s;"

        #SQLITE
        sql = "DELETE FROM notas WHERE id = ?;"

        try:
            params = (note.Id,)
        
            context = Context()

            context.Cursor.execute(sql, params)
            context.db.commit()

            result = context.Cursor.rowcount
        except Exception as ex:
            result = 0
        return result
    
    def GetById(self, id):
        #MYSQL
        # sql = "SELECT n.Id, n.titulo, n.descripcion, n.usuarioId FROM notas n WHERE n.id = %s;"

        #SQLITE
        sql = "SELECT n.Id, n.titulo, n.descripcion, n.usuarioId FROM notas n WHERE n.id = ?;"
        
        params = (id,)
    
        context = Context()

        context.Cursor.execute(sql, params)
           
        resultDb = context.Cursor.fetchone()

        return resultDb

    def GetByUser(self, userId):
        #MYSQL
        # sql = "SELECT n.Id, n.titulo, n.descripcion, n.usuarioId FROM notas n WHERE n.usuarioId = %s;"

        #SQLITE
        sql = "SELECT n.Id, n.titulo, n.descripcion, n.usuarioId FROM notas n WHERE n.usuarioId = ?;"
        
        params = (userId,)
    
        context = Context()

        context.Cursor.execute(sql, params)
           
        resultDb = context.Cursor.fetchall()

        return resultDb



import datetime
import hashlib
from dataAcces.context import Context

class UserDA:

    def Insert(self, user):
        #MYSQL
        # sql = "INSERT INTO usuarios VALUES(NULL, %s, %s, %s, %s, %s);"

        #SQLITE
        sql = "INSERT INTO usuarios VALUES(NULL, ?, ?, ?, ?, ?);"

        try:
            #Cifrar contraseña
            hashedPass = hashlib.sha256()
            hashedPass.update(user.Password)

            fecha = datetime.date.today()
            params = (user.Name, user.LastName, user.Email, hashedPass.hexdigest(), fecha)
        
            context = Context()

            context.Cursor.execute(sql, params)
            context.db.commit()

            result = context.Cursor.rowcount
        except Exception as ex:
            result = 0
        return result
    
    def GetByEmailPassword(self, email, password):
        #MYSQL
        # sql = "SELECT id, nombres, apellidos, email FROM usuarios WHERE email = %s AND password = %s;"

        #SQLITE
        sql = "SELECT id, nombres, apellidos, email FROM usuarios WHERE email = ? AND password = ?;"

        
        #Cifrar contraseña
        hashedPass = hashlib.sha256()
        hashedPass.update(password)

        params = (email, hashedPass.hexdigest())
    
        context = Context()

        context.Cursor.execute(sql, params)
           
        resultDb = context.Cursor.fetchone()

        return resultDb




import re
from businessLogic.validationException import ValidationException
from dataAcces.userDA import UserDA
from entites.user import User

class UserManager:
    
    def SignUp(self, user: User, passwordConfirm: bytes):
        da = UserDA()
        
        self.__checkValidEmail(user.Email)
        self.__checkPasswordConfirm(user.Password,passwordConfirm)
        
        return da.Insert(user)

    def Login(self, email: str, password: str):
        da = UserDA()
        
        resultDb = da.GetByEmailPassword(email, password)

        if(resultDb != None):
            return User(resultDb[1],resultDb[2],resultDb[3],id=resultDb[0])
        else:
            return None

    def __checkPasswordConfirm(self, password: bytes, passwordConfirm: bytes):
        if(password != passwordConfirm):
            raise ValidationException("Debe ingresar una contraseña que coincida con el campo de confirmación.", field="Password")

    def __checkValidEmail(self, email: str):
        regex = '^(\w|\.|\_|\-)+[@](\w|\_|\-|\.)+[.]\w{2,3}$'
        
        if(not re.search(regex, email)):
            raise ValidationException("Debe ingresar un email válido.", field="Email")



def singleton(cls):
    #Decorador para patron singleton

    instances = dict()

    def wrap(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args,**kwargs)

        return instances[cls]

    return wrap


from typing import Optional


class User:

    def __init__(self, 
                names: str, 
                lastNames: str,
                email: str, 
                password: Optional[bytes] = None,
                id: Optional[int] = None):
        self.Id = id
        self.Name = names
        self.LastName = lastNames
        self.Email = email
        self.Password = password



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh1:
    long_description = fh1.read()

#with open("CHANGELOG.txt", "r", encoding="utf-8") as fh2:
#    change_log = fh2.read()

VERSION = '1.0.0'

setup(name='sample-backend',
    version=VERSION,
    description='Sample backend for todo-list application',
    long_description=long_description,
    long_description_content_type="text/markdown",
    #change_log = change_log,
    url='https://github.com/malonejv/py-sample-backend',  
    project_urls={
        "Bug Tracker": "https://github.com/malonejv/py-sample-backend/issues",
    },
    author='Javier López Malone',
    author_email='malonejv@gmail.com',
    license='MIT', 
    keywords=['todolist','todo'], 
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.9",
)



import os
# from common.singleton import singleton
from infrastructure.singleton import singleton

class ConfigKeys:
    ENABLE_THEME = "EnableTheme"
    LOGIN_USER = "User"
    LOGIN_PASSWORD = "Password"

@singleton
class ConfigHelper:
    
    __config = dict()

    def __init__(self):
        #Ruta absoluta
        configPath = os.path.abspath("../config.ini")
        if not os.path.isfile(configPath):
            configPath=os.path.abspath("./section22-TkinterMySql/config.ini")


        f = open(configPath,"r")

        lines = f.readlines()
        for line in lines:
            if(not line.startswith("#")):
                key,value = line.strip('\n\r\t').split("=")
                self.__config.setdefault(key,value)

        f.close()

    @classmethod
    def GetConfig(cls):
        return cls.__config

    @classmethod
    def GetValue(cls, key):
        return cls.__config.get(key)




# import mysql.connector
import sqlite3
import os

class Context:

    #SQLITE
    def __init__(self) -> None:
        
        #Ruta absoluta
        databasePath = os.path.abspath("./sample.db")
        if not os.path.isfile(databasePath):
            databasePath=os.path.abspath("../../database/sample.db")

        self.__db = sqlite3.connect(databasePath)
        self.db = self.__db

        self.__cursor = self.__db.cursor()
        self.Cursor = self.__cursor

    def CreateConnection(self):
        return [self.__db, self.__cursor]

    # #MYSQL
    # def __init__(self) -> None:
    #     self.__db = mysql.connector.connect(
    #         host= "localhost",
    #         user = "root",
    #         passwd = "sql.123",
    #         port = 3306,
    #         database = "test"
    #     )
    #     self.db = self.__db

    #     self.__cursor = self.__db.cursor(buffered=True)
    #     self.Cursor = self.__cursor

    # def CreateConnection(self):
    #     return [self.__db, self.__cursor]